/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>



#define VER     "0.1"
#define HEADSZ  444



void pcanywhere_dec(unsigned char *data, int size);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE            *in,
                    *out;
    unsigned int   len;
    unsigned char   *buff;
    struct  stat    xstat;


    setbuf(stdout, NULL);

    fputs("\n"
        "PC Anywhere configuration files decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\n"
            "Usage: %s <input_file> <output_file>\n"
            "\n"
            "input_file is ANY of the files available in the configuration folder of PC\n"
            "Anywhere, usually located in:\n"
            "\n"
            "  c:\\windows\\All Users\\Application Data\\Symantec\\pcAnywhere\n"
            "\n"
            "If you have problems finding this folder simply search for folders called\n"
            "Symantec or pcAnywhere in your Windows directory\n"
            "\n", argv[0]);
        exit(1);
    }

    printf("- open input file \"%s\"\n", argv[1]);
    in = fopen(argv[1], "rb");
    if(!in) std_err();

    printf("- create output file \"%s\"\n", argv[2]);
    out = fopen(argv[2], "wb");
    if(!out) std_err();

    fstat(fileno(in), &xstat);
    len = xstat.st_size;

    buff = malloc(len);
    if(!buff) std_err();

    if(len >= HEADSZ) { // fixed header
        fread(buff, HEADSZ, 1, in);
        pcanywhere_dec(buff, HEADSZ);
        fwrite(buff, HEADSZ, 1, out);
        len -= HEADSZ;
    }

    fread(buff, len, 1, in);
    pcanywhere_dec(buff, len);
    fwrite(buff, len, 1, out);

    fclose(in);
    fclose(out);
    fputs(
        "- finished\n"
        "- now open the output file with an hex editor and search the passwords or the\n"
        "  other informations you need\n"
        "\n", stdout);
    return(0);
}



void pcanywhere_dec(unsigned char *data, int size) {
    size--;
    if(size <= 0) return;
    data += size;
    while(size--) {
        *data = (*data ^ *(data - 1)) ^ size;
        data--;
    }
    *data ^= 0xab;
}



void std_err(void) {
    perror("\nError");
    exit(1);
}

